from abc import ABC, abstractmethod
from typing import List, Dict


# Single Responsibility Principle (SRP): Class for handling user-related functionalities.
class User:
    def __init__(self, user_id: str, name: str, location: str):
        self.user_id = user_id
        self.name = name
        self.location = location

    def update_location(self, new_location: str):
        self.location = new_location


# Store Class for storing store information, stores information like store name, location, and items.
class Store:
    def __init__(self, store_id: str, name: str, location: str, items: Dict[str, float]):
        self.store_id = store_id
        self.name = name
        self.location = location
        self.items = items

    def get_items(self) -> Dict[str, float]:
        return self.items

    def add_item(self, item_name: str, price: float):
        self.items[item_name] = price


# Open/Closed Principle (OCP): Payment system can be extended by adding new payment types.
class PaymentMethod(ABC):
    @abstractmethod
    def pay(self, amount: float) -> bool:
        pass


class CreditCardPayment(PaymentMethod):
    def pay(self, amount: float) -> bool:
        print(f"Paying {amount} with Credit Card")
        return True


class PayPalPayment(PaymentMethod):
    def pay(self, amount: float) -> bool:
        print(f"Paying {amount} with PayPal")
        return True


# Liskov Substitution Principle (LSP): User can use different payment methods.
class ShoppingCart:
    def __init__(self):
        self.items = []

    def add_item(self, store: Store, item_name: str):
        if item_name in store.items:
            self.items.append((item_name, store.items[item_name]))
        else:
            print(f"Item {item_name} not available in store.")

    def total_price(self) -> float:
        return sum(item[1] for item in self.items)

    def checkout(self, payment_method: PaymentMethod):
        total = self.total_price()
        payment_method.pay(total)
        print(f"Total paid: {total}")


# Interface Segregation Principle (ISP): Small interfaces for location services.
class LocationService(ABC):
    @abstractmethod
    def find_nearby_stores(self, user_location: str) -> List[Store]:
        pass


class StoreLocator(LocationService):
    def __init__(self, stores: List[Store]):
        self.stores = stores

    def find_nearby_stores(self, user_location: str) -> List[Store]:
        # Simple logic to return stores near the user based on location
        return [store for store in self.stores if store.location == user_location]


# Dependency Inversion Principle (DIP): High-level modules depend on abstractions.
class LocationBasedShop:
    def __init__(self, user: User, store_locator: LocationService):
        self.user = user
        self.store_locator = store_locator

    def show_nearby_stores(self):
        stores = self.store_locator.find_nearby_stores(self.user.location)
        print(f"Stores near {self.user.name}:")
        for store in stores:
            print(f"{store.name} - Items: {store.get_items()}")

    def shop_at_store(self, store: Store, item_name: str, payment_method: PaymentMethod):
        cart = ShoppingCart()
        cart.add_item(store, item_name)
        cart.checkout(payment_method)


# Authentication Providers: Handle user authentication through SSO or cookies.
class AuthenticationProvider(ABC):
    @abstractmethod
    def authenticate(self, user_id: str) -> bool:
        pass


class GoogleAuthProvider(AuthenticationProvider):
    def authenticate(self, user_id: str) -> bool:
        print(f"Authenticating user {user_id} via Google SSO...")
        return True  # Mock successful authentication


class FacebookAuthProvider(AuthenticationProvider):
    def authenticate(self, user_id: str) -> bool:
        print(f"Authenticating user {user_id} via Facebook SSO...")
        return True  # Mock successful authentication


class CookieAuthProvider(AuthenticationProvider):
    def authenticate(self, user_id: str) -> bool:
        print(f"Authenticating user {user_id} via Cookies...")
        return True  # Mock successful authentication


# Authentication Manager to handle authentication flow
class AuthManager:
    def __init__(self, provider: AuthenticationProvider):
        self.provider = provider

    def login(self, user: User):
        if self.provider.authenticate(user.user_id):
            print(f"User {user.name} authenticated successfully!")
        else:
            print(f"Authentication failed for user {user.name}.")


# Example Usage
if __name__ == "__main__":
    # Initialize stores
    stores = [
        Store("1", "Store A", "Location1", {"Item1": 10.0, "Item2": 20.0}),
        Store("2", "Store B", "Location2", {"Item1": 15.0, "Item3": 25.0}),
    ]

    # Create a user
    user = User("1", "Alice", "Location1")

    # Authentication flow
    google_auth = GoogleAuthProvider()
    auth_manager = AuthManager(google_auth)
    auth_manager.login(user)

    # Location and shopping flow
    store_locator = StoreLocator(stores)
    location_based_shop = LocationBasedShop(user, store_locator)

    # Show nearby stores
    location_based_shop.show_nearby_stores()

    # Shopping and checkout
    store = stores[0]  # Store A
    location_based_shop.shop_at_store(store, "Item1", CreditCardPayment())
